# -*- coding: utf-8 -*-

"""{{ cookiecutter.site_name }} application factories."""

import os
import sys

from invenio_base.app import create_app_factory
from invenio_base.wsgi import create_wsgi_factory
from invenio_config import create_conf_loader

from . import config

env_prefix = 'APP'

config_loader = create_conf_loader(config=config, env_prefix=env_prefix)

instance_path = os.getenv(env_prefix + '_INSTANCE_PATH') or \
    os.path.join(sys.prefix, 'var', '{{ cookiecutter.package_name }}-instance')
"""Instance path for Invenio.

Defaults to ``<env_prefix>_INSTANCE_PATH`` or if environment variable is not
set ``<sys.prefix>/var/<app_name>-instance``.
"""

static_folder = os.getenv(env_prefix + '_STATIC_FOLDER') or \
    os.path.join(instance_path, 'static')
"""Static folder path.

Defaults to ``<env_prefix>_STATIC_FOLDER`` or if environment variable is not
set ``<sys.prefix>/var/<app_name>-instance/static``.
"""

create_api = create_app_factory(
    '{{ cookiecutter.package_name }}',
    config_loader=config_loader,
    blueprint_entry_points=['invenio_base.api_blueprints'],
    extension_entry_points=['invenio_base.api_apps'],
    converter_entry_points=['invenio_base.api_converters'],
    instance_path=instance_path,
)


create_app = create_app_factory(
    '{{ cookiecutter.package_name }}',
    config_loader=config_loader,
    blueprint_entry_points=['invenio_base.blueprints'],
    extension_entry_points=['invenio_base.apps'],
    converter_entry_points=['invenio_base.converters'],
    wsgi_factory=create_wsgi_factory({'/api': create_api}),
    instance_path=instance_path,
    static_folder=static_folder,
)



# -*- coding: utf-8 -*-

"""{{ cookiecutter.site_name }} Invenio WSGI application."""

from __future__ import absolute_import, print_function

from .factory import create_app

application = create_app()



# -*- coding: utf-8 -*-

"""{{ cookiecutter.site_name }} base Invenio configuration."""

from __future__ import absolute_import, print_function

from flask import Blueprint, render_template

blueprint = Blueprint(
    '{{ cookiecutter.package_name }}',
    __name__,
    template_folder='templates',
    static_folder='static',
)


@blueprint.route("/")
def index():
    """Home Page."""
    return render_template('{{ cookiecutter.package_name }}/index.html',
                           module_name='{{ cookiecutter.package_name }}')



# -*- coding: utf-8 -*-

"""{{ cookiecutter.site_name }} base Invenio configuration."""

from __future__ import absolute_import, print_function


# Identity function for string extraction
def _(x):
    return x

# Default language and timezone
BABEL_DEFAULT_LANGUAGE = 'en'
BABEL_DEFAULT_TIMEZONE = 'Europe/Zurich'
I18N_LANGUAGES = [
]

HEADER_TEMPLATE = "invenio_theme/header.html"
BASE_TEMPLATE = "invenio_theme/page.html"
COVER_TEMPLATE = "invenio_theme/page_cover.html"
SETTINGS_TEMPLATE = "invenio_theme/settings/content.html"

# WARNING: Do not share the secret key - especially do not commit it to
# version control.
SECRET_KEY = "{{ cookiecutter.secret_key }}"

# Theme
THEME_SITENAME = _("{{ cookiecutter.site_name }}")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization or
# submit itself to any jurisdiction.

"""Test wsgi application."""

from __future__ import absolute_import, print_function, unicode_literals

from flask import Flask, request

from invenio_base.wsgi import create_wsgi_factory, wsgi_proxyfix


def test_create_wsgi_factory():
    """Test wsgi factory creation."""
    api = Flask('api')

    @api.route("/")
    def apiview():
        return "api"

    app = Flask('app')

    @app.route("/")
    def appview():
        return "app"

    # Test factory creation
    factory = create_wsgi_factory({'/api': lambda **kwargs: api})
    app.wsgi_app = factory(app)

    with app.test_client() as client:
        assert client.get('/').status_code == 200
        assert b'app' in client.get('/').data
        assert client.get('/api/').status_code == 200
        assert b'api' in client.get('/api/').data


def test_proxyfix():
    """Test wsgi factory creation."""
    app = Flask('app')
    app.config['WSGI_PROXIES'] = 2

    @app.route("/")
    def appview():
        return str(request.remote_addr)

    # Test factory creation
    app.wsgi_app = wsgi_proxyfix()(app, num_proxies=2)
    e = {'REMOTE_ADDR': '1.2.3.4'}

    with app.test_client() as client:
        h = {'X-Forwarded-For': '5.6.7.8, 4.3.2.1, 8.7.6.5'}
        assert client.get('/', headers=h, environ_base=e).data == b'4.3.2.1'



# -*- coding: utf-8 -*-

"""{{ cookiecutter.site_name }} base Invenio configuration."""

from __future__ import absolute_import, print_function

from flask_celeryext import create_celery_app

from .factory import create_app

celery = create_celery_app(create_app())



# -*- coding: utf-8 -*-

"""Version information for {{ cookiecutter.site_name }}.

This file is imported by ``{{ cookiecutter.package_name }}.__init__``,
and parsed by ``setup.py``.
"""

__version__ = "0.1.0.dev0"



# -*- coding: utf-8 -*-

"""{{ cookiecutter.site_name }} base Invenio configuration."""

from __future__ import absolute_import, print_function

from invenio_base.app import create_cli

from .factory import create_app

cli = create_cli(create_app=create_app)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2012, 2013, 2014, 2015 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio application loader.

The ``inveniomanage`` command
-----------------------------
Invenio-Base installs the ``inveniomanage`` command. By default only three
subcommands are available:

.. code-block:: console

   $ inveniomanage --help
   Usage: inveniomanage [OPTIONS] COMMAND [ARGS]...

     Command Line Interface for Invenio.

   Options:
     -a, --app TEXT        The application to run.
     --debug / --no-debug  Enable or disable debug mode.
     --help                Show this message and exit.

   Commands:
     run              Run development server.
     shell            Run shell in the app context.
     instance create  Create a new Invenio instance from template.


The ``run`` and ``shell`` commands only works if you have specified the
``--app`` option or the ``FLASK_APP`` environment variable. See
`Flask <http://flask.pocoo.org/docs/dev/cli/>`_ documentation for further
information.


Creating a new Invenio instance
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
The ``instance create`` subcommand helps you bootstrap a minimal Invenio
application:

.. code-block:: console

   $ inveniomanage instance create mysite
   $ find mysite


You can install the newly created site using the provided ``setup.py`` script:

.. code-block:: console

   $ cd mysite
   $ python setup.py install

After installation you can use ``mysite`` command to manage you instance.

.. code-block:: console

   $ mysite --help
   ...

"""

from __future__ import absolute_import, print_function

from .app import create_app_factory, create_cli
from .version import __version__
from .wsgi import create_wsgi_factory

__all__ = (
    '__version__',
    'create_app_factory',
    'create_cli',
    'create_wsgi_factory',
)



# -*- coding: utf-8 -*-

"""{{ cookiecutter.site_name }} Invenio instance."""

import os

from setuptools import find_packages, setup

# Get the version string. Cannot be done with import!
version = {}
with open(os.path.join('{{ cookiecutter.package_name }}',
                       'version.py'), 'rt') as fp:
    exec(fp.read(), version)

setup(
    name='{{ cookiecutter.site_name }}',
    version=version['__version__'],
    description=__doc__,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={
        'console_scripts': [
            '{{ cookiecutter.package_name }} = '
            '{{ cookiecutter.package_name }}.cli:cli',
        ],
        'invenio_base.blueprints': [
            '{{ cookiecutter.package_name }} = '
            '{{ cookiecutter.package_name }}.views:blueprint',
        ],
    },
    install_requires=[
        'Flask>=0.11.1',
        'invenio-assets>=1.0.0a4',
        'invenio-base>=1.0.0a11',
        'invenio-config>=1.0.0a1',
        'invenio-db>=1.0.0a9',
        'invenio-indexer>=1.0.0a2',
        'invenio-jsonschemas>=1.0.0a2',
        'invenio-marc21>=1.0.0a1',
        'invenio-oaiserver>=1.0.0a1',
        'invenio-pidstore>=1.0.0a6',
        'invenio-records-rest>=1.0.0a11',
        'invenio-records-ui>=1.0.0a4',
        'invenio-search-ui>=1.0.0a2',
        'invenio-search>=1.0.0a5',
        'invenio-theme>=1.0.0a10',
    ],
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization or
# submit itself to any jurisdiction.

"""Test cli application."""

from __future__ import absolute_import, print_function, unicode_literals

import importlib
import os
from subprocess import call

import pkg_resources
from click.testing import CliRunner
from mock import Mock, patch

from invenio_base.cmd import instance


def test_instance_create():
    """Test ``instance create`` command."""
    runner = CliRunner()

    # Missing arg
    result = runner.invoke(instance, ['create'])
    assert result.exit_code != 0

    # With arg
    with runner.isolated_filesystem():
        result = runner.invoke(instance, ['create', 'mysite'])
        assert result.exit_code == 0


def test_instance_create_created_instance():
    """Test ``instance create`` command checking the resulting instance."""
    runner = CliRunner()

    # Missing arg
    result = runner.invoke(instance, ['create'])
    assert result.exit_code != 0

    # With arg
    with runner.isolated_filesystem():
        site_name = 'mysite2'
        result = runner.invoke(instance, ['create', site_name])
        assert result.exit_code == 0

        cwd = os.getcwd()
        path_to_folder = os.path.join(cwd, site_name)
        os.chdir(path_to_folder)

        if os.getenv('REQUIREMENTS') == 'devel':
            assert call(
                ['pip', 'install', '-r', 'requirements-devel.txt']
            ) == 0

        assert call(['pip', 'install', '-e', '.']) == 0
        assert pkg_resources.get_distribution(site_name)

        app = importlib.import_module(site_name + '.factory').create_app()
        with app.app_context():
            assert app.name == site_name

        assert call([site_name, '--help']) == 0
        assert call(['pip', 'uninstall', site_name, '-y']) == 0

        os.chdir(cwd)


def test_list_entry_points():
    """Test listing of entry points."""
    mock_working_set = pkg_resources.WorkingSet(entries=[])
    dist = pkg_resources.get_distribution('invenio-base')
    mock_working_set.add(dist)

    with patch('invenio_base.cmd.working_set', new=mock_working_set):
        runner = CliRunner()

        # Test select an existing entry point
        result = runner.invoke(
            instance, ['entrypoints', '-e', 'console_scripts'])
        assert result.exit_code == 0
        lines = result.output.splitlines()
        assert lines[0] == 'console_scripts'
        assert lines[1] == '  inveniomanage = invenio_base.cli:cli'

        # Test no entry point matching
        result = runner.invoke(
            instance, ['entrypoints', '-e', 'nothing_here'])
        assert result.exit_code == 0
        assert result.output == ""

        # By default we only show entry points groups starting with "invenio"
        dist.get_entry_map = Mock(return_value={
            'invenio_base.apps': {
                'myapp': 'myapp = myapp:MyApp',
                'app1': 'app1 = app1:MyApp'},
            'invenio_base.api_apps': {
                'myapi': 'myapi = myapi:MyApp'},
            'console_scripts': {
                'mycli': 'mycli = cli:main'},
        })
        result = runner.invoke(instance, ['entrypoints', ])
        assert result.exit_code == 0
        print(result.output.splitlines())
        lines = result.output.splitlines()
        assert lines[0] == 'invenio_base.api_apps'
        assert lines[1] == '  myapi = myapi:MyApp'
        assert lines[2] == 'invenio_base.apps'
        assert lines[3] == '  app1 = app1:MyApp'
        assert lines[4] == '  myapp = myapp:MyApp'



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization or
# submit itself to any jurisdiction.

"""Application bootstraping."""

from __future__ import absolute_import, print_function

import click
from cookiecutter.main import cookiecutter
from pkg_resources import resource_filename, working_set


@click.group()
def instance():
    """Instance commands."""


@instance.command('create')
@click.argument('name')
def create(name):
    """Create a new Invenio instance from template."""
    path = resource_filename(__name__, "cookiecutter-invenio-base")

    result = cookiecutter(path, no_input=True,
                          extra_context={
                              "site_name": name,
                              "secret_key": generate_secret_key()
                          })
    click.secho("Created instance...", fg="green")
    return result


@instance.command('entrypoints')
@click.option(
    '-e', '--entry-point', default=None, metavar='ENTRY_POINT',
    help='Entry point group name (e.g. invenio_base.apps)')
def list_entrypoints(entry_point):
    """List defined entry points."""
    found_entry_points = {}
    for dist in working_set:
        entry_map = dist.get_entry_map()
        for group_name, entry_points in entry_map.items():
            # Filter entry points
            if entry_point is None and \
               not group_name.startswith('invenio'):
                continue
            if entry_point is not None and \
               entry_point != group_name:
                continue

            # Store entry points.
            if group_name not in found_entry_points:
                found_entry_points[group_name] = []
            for ep in entry_points.values():
                found_entry_points[group_name].append(str(ep))

    for ep_group in sorted(found_entry_points.keys()):
        click.secho("{0}".format(ep_group), fg='green')
        for ep in sorted(found_entry_points[ep_group]):
            click.echo("  {0}".format(ep))


def generate_secret_key():
    """Generate secret key."""
    import string
    import random

    rng = random.SystemRandom()
    return ''.join(
        rng.choice(string.ascii_letters + string.digits)
        for dummy in range(0, 256)
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization or
# submit itself to any jurisdiction.

"""WSGI application factory for Invenio."""

from __future__ import absolute_import, print_function

from werkzeug.contrib.fixers import ProxyFix
from werkzeug.wsgi import DispatcherMiddleware


def create_wsgi_factory(mounts_factories):
    """Create a WSGI application factory.

    Usage example:

    .. code-block:: python

       wsgi_factory = create_wsgi_factory(create_app, {'/api': create_api})

    :param app_factory: Flask application factory for the default application.
    :param mounts_factories: Dictionary of mount points per application
        factory.
    """
    def create_wsgi(app, **kwargs):
        mounts = {
            mount: factory(**kwargs)
            for mount, factory in mounts_factories.items()
        }
        return DispatcherMiddleware(app.wsgi_app, mounts)
    return create_wsgi


def wsgi_proxyfix(factory=None):
    """Fix REMOTE_ADDR based on X-Forward-For headers.

    Note you must set ``WSGI_PROXIES`` to the correct number of proxies,
    otherwise you application is susceptible to malicious attacks.
    """
    def create_wsgi(app, **kwargs):
        wsgi_app = factory(app, **kwargs) if factory else app.wsgi_app
        if app.config.get('WSGI_PROXIES'):
            return ProxyFix(wsgi_app, num_proxies=app.config['WSGI_PROXIES'])
        return wsgi_app
    return create_wsgi
